/*////////////////////////////////////////////////////////////////////////
Copyright (c) 1992 Electrotechnical Laboratry (ETL)

Permission to use, copy, modify, and distribute this material 
for any purpose and without fee is hereby granted, provided 
that the above copyright notice and this permission notice 
appear in all copies, and that the name of ETL not be 
used in advertising or publicity pertaining to this 
material without the specific, prior written permission 
of an authorized representative of ETL.
ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
/////////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	history.c (news history scanner)
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:
History:
	920921	extracted out of xhist.c
///////////////////////////////////////////////////////////////////////*/
#ifndef INCLUDE_DONE

/* select dbm/ndbm/dbz and CNEWS/BNEWS */
#ifdef WITHNNTP
#include "common.h"
#else
#include <ndbm.h>
#include <sys/types.h>
#include <sys/stat.h>
#endif

#include <stdio.h>
#endif

#if defined(CNEWS) || defined(Cnews)
int IS_CNEWS = 1;
#else
int IS_CNEWS = 0;
#endif


#if defined(Cnews) || defined(CNEWS) || !defined(_DBM_RDONLY) /* dbm or dbz */
#define DBMINIT(name)	dbminit(name)
#define	DBMCLOSE()	dbmclose()
#define DBMFETCH(key)	fetch(key)
datum fetch();

#else /* maybe ndbm */
DBM *Hdbm;
#define	DBMINIT(name)	((Hdbm = dbm_open(name,0,0)) == 0 ? -1:0)
#define	DBMCLOSE()	dbm_close(Hdbm)
#define DBMFETCH(key)	dbm_fetch(Hdbm,key)

#endif /* DBM selection */


historyByMessageID(history,mid,line,linesize)
	char *history,*mid,*line;
{	FILE *HFP;	/* History File		*/
	datum key,data;
	int offset;
	unsigned char *cp,midb[1000];

	if( DBMINIT(history) < 0 )
		return 0;

	mid = (char*)strcpy(midb,mid);
	if( IS_CNEWS ){
		if( cp = (unsigned char *)rindex(mid,'@') )
			xhist_tolowers(cp);
	}else	xhist_tolowers(mid);

	key.dptr = mid;
	key.dsize = strlen(key.dptr) + 1;
	data.dptr = 0;
	data = DBMFETCH(key);
	DBMCLOSE();

	if(data.dptr == 0)
		return -1;

	if(cp = (unsigned char*)data.dptr){
		if((HFP = fopen(history,"r")) == NULL)
			return 0;
		offset = cp[0]<<24 | cp[1]<<16 | cp[2]<<8 | cp[3];
		fseek(HFP,offset,0);
		fgets(line,linesize,HFP);
		fclose(HFP);
		linesize = strlen(line);
		if(linesize && line[linesize-1] == '\n')
			line[linesize-1] = 0;
		return 1;
	}
	return 0;
}
historyLastLine(history,line)
	char *history,*line;
{	FILE *fp;
	char buf[2048],*bp;

	*line = 0;
	fp = fopen(history,"r");
	if( fp != NULL ){
		fseek(fp,-sizeof(buf)+2,2);
/* 92-07-24 THE LONGEST LIFE BUG MEMORIAL !!
		fscanf(fp,"%[^\377]",buf); */

		fgets(buf,sizeof(buf),fp);
		while( fgets(buf,sizeof(buf),fp) != NULL )
			if( buf[strlen(buf)-1] == '\n' ){
				buf[strlen(buf)-1] = 0;
				strcpy(line,buf);
			}
		fclose(fp);
	}
}
history_bsearch(fp,date)
	FILE *fp;
{	int fsize,offs,offset;
	int itime;
	char line[1000];

	fsize = FileSize(fp);
	offs = fsize / 2;
	fseek(fp,offs,0);
	offset = 0;

	while( 80 < offs ){
		fgets(line,sizeof(line),fp);
		offset = ftell(fp);
		fgets(line,sizeof(line),fp);
		itime = decomp_history_line(line,0,0);
		if( itime <= 0 )
			return fsize;

		offs /= 2;
		if( date < itime )
			fseek(fp,-offs,1);
		else	fseek(fp, offs,1);
	}
	return offset;
}

fseek_history_bydate(fp,date)
	FILE *fp;
{	int offset;

	offset = history_bsearch(fp,date);
	fseek(fp,offset,0);
}

/*
 *	Decomposit 4-formats of history line
 */
decomp_history_line(line,id,xref)
	char *line,*id,*xref;
{	int day,month,year,hour,minute,itime;
	char tid[1000],txref[5000],*dp;
	int nitem,optdate;

	if( id == 0 ) id = tid;
	if( xref == 0 ) xref = txref;

	year = 0;
	*xref = 0;

	nitem = sscanf(line,"%s %d/%d/%d %d:%d %d %[^\n\r]",
		id,&month,&day,&year,&hour,&minute,&optdate,xref);
	if( 7 <= nitem ){
		itime = TimeGm(year,month,day,hour,minute,0);
		return itime;
	}

	nitem = sscanf(line,"%s %d/%d/%d %d:%d %[^\n\r]",
		id,&month,&day,&year,&hour,&minute,xref);
	if( 7 == nitem ){
		itime = TimeGm(year,month,day,hour,minute,0);
		return itime;
	}

	/* new Bnews or Cnews */
	nitem = sscanf(line,"%s %d %[^\n\r]",id,&itime,xref);
	if( nitem == 3 ){
		if( *xref != '~' )
			return itime;
		if( dp = (char*)index(xref,'\t') )
			strcpy(xref,dp+1);
		else	*xref = 0;
		return itime;
	}

	return 0;
}

/*
#include <time.h>
TimeGm(year,month,mday,hour,min,sec){
	static struct tm tm;

	tm.tm_year = year;
	tm.tm_mon = month - 1;
	tm.tm_mday = mday;
	tm.tm_hour = hour;
	tm.tm_min = min;
	tm.tm_sec = sec;
	return timegm(&tm);
}
*/
/*
 *	GMT-TIME TO CLOCK (unreliable coding ;-)
 */
static int month_base[] =
	{ 0,31,59,90,120,151,181,212,243,273,304,334,1000 };
TimeGm(year,month,mday,hour,min,sec){
	int md;

	if( month < 1 || 12 < month )
		return 0;
	year -= 70; month -= 1; mday -= 1;
	md = ((((year-2) % 4) == 0) && (1 < month)) ? 1 : 0;
	return ( ( ( year*365 + month_base[month] + md + mday + (year+1)/4
	    	    ) * 24 + hour ) * 60 + min) * 60 + sec;
}

FileSize(fp)
	FILE *fp;
{	static struct stat status;

	if( fstat(fileno(fp),&status) < 0 )
		return 0;
	else	return status.st_size;
}
FileMtime(fp)
	FILE *fp;
{	static struct stat status;

	if( fstat(fileno(fp),&status) < 0 )
		return 0;
	else	return status.st_mtime;
}

#include <ctype.h>
xhist_tolowers(str)
	char *str;
{	register char *s;

	for(s = str; *s; s++)
		if(isupper(*s))
			*s = tolower(*s);
}

char XHIST_LASTLINE_COMMAND[8] = "$";
